% TEMPLATE FOR THESES
% Created by: Daniel Rotter <daniel.rotter@itsec.uni-hannover.de>
% Based on previous work by: André Miede, Ivo Pletikosić, Florian Farke, and Maike Raphael.
% Updated: 2024-Feb-09
%
% Contact your advisor in case you have questions.

\NeedsTeXFormat{LaTeX2e}
\ProvidesClass{internal/usp_thesis}[2024/02/09 LUH USP Thesis]


% ******************************************************************************
% 0. General setup of this class
% ******************************************************************************

% Hide certain warnings ...
\RequirePackage{silence}
% ...  about ignored (i.e., redundant) package options
\WarningFilter{hyperref}{Option `hyperfootnotes' has already been used,} % setting the option has no effect
\WarningFilter{hyperref}{Option `bookmarks' has already been used,} % setting the option has no effect
% ... incompatibilities between KOMA-Script classes and the titlesec package (which is part of classicthesis)
\WarningFilter{titlesec}{Non standard sectioning command}
% ... incompatibilities between KOMA-Script classes and the listings package (though in theory there should be no incompatibilities)
\WarningFilter{scrhack}{unknown \lstlistoflistings definition found!} % Maybe you are using a unsupported listings version.

% Definition of error messages that inform abut the wrong usage of this classes options
\newcommand{\languageerror}{
    \ClassError{usp_thesis}{Cannot set thesis language to both 'english' and 'german'}{Remove either 'english' or 'german' from the options list when loading this class}
    \stop  % Stop LaTeX compilation if more than one language is specified
}
\newcommand{\thesistypeerror}{
    \ClassError{usp_thesis}{Cannot set thesis type to both 'bachelor' and 'master'}{Remove either 'bachelor' or 'master' from the options list when loading this class}
    \stop  % Stop LaTeX compilation if more than one language is specified
}

% Define mutually exclusive options for language and thesis type
\DeclareOption{german}{
    \ifdefined\uspenglish \languageerror{} \fi
    \def\uspgerman{}
    \PassOptionsToClass{german}{scrreprt}
    \PassOptionsToPackage{main=ngerman,english}{babel}
    \PassOptionsToPackage{de-DE}{datetime2}
    \PassOptionsToPackage{locale=DE}{siunitx}
}
\DeclareOption{english}{
    \ifdefined\uspgerman \languageerror{} \fi
    \def\uspenglish{}
    \PassOptionsToClass{\CurrentOption}{scrreprt}
    \PassOptionsToPackage{main=english,ngerman}{babel}
    \PassOptionsToPackage{en-US, de-DE}{datetime2}
    \PassOptionsToPackage{locale=US}{siunitx}
}

\DeclareOption{bachelor}{
    \ifdefined\uspmaster \thesistypeerror{} \fi
    \def\uspbachelor{}
}
\DeclareOption{master}{
    \ifdefined\uspbachelor \thesistypeerror{} \fi
    \def\uspmaster{}
}

% Process all options and warn if any options are not used
\DeclareOption*{\OptionNotUsed}
\ProcessOptions*\relax
\LoadClass[
    twoside         = true,         % Use two-sided printing (i.e., inner/outer margins instead of left/right margins)
    openright,                      % The first page (of each chapter) is the "right side of a double page"
    titlepage       = firstiscover, % First invocation of \maketitle creates a cover page
    numbers         = noenddot,     % Do not use trailing point for section numbers
    cleardoublepage = empty,        % Hide header/footer on interleaf pages
    abstract        = true,         % Use the text "Abstract" as title for the abstract environment
    fontsize        = 11pt
    ]{scrreprt}
    
% For some reason there are warnings about unused global options when the following options are specified at the time the 'scrreprt' class is loaded
\KOMAoptions{
    headinclude     = true,         % Include header in text body size calculations
    footinclude     = true,         % Include footer in text body size calculations
    BCOR            = 10mm,         % Binding correction. Assume that 10 mm of paper are "lost" due to binding
    paper           = a4,
}

\RequirePackage{ifthen}

% Make sure that both the language and the thesis type have been set
\ifthenelse{\isundefined{\uspgerman} \AND \isundefined{\uspenglish}}{
    \ClassError{usp_thesis}{Must set thesis language to either 'english' or 'german'}{Add either 'english' or 'german' to the options list when loading this class}
    \stop  % Stop LaTeX compilation if no language is specified
}{}
\ifthenelse{\isundefined{\uspbachelor} \AND \isundefined{\uspmaster}}{
    \ClassError{usp_thesis}{Must set thesis type to either 'bachelor' and 'master'}{Add either 'bachelor' or 'master' to the options list when loading this class}
    \stop  % Stop LaTeX compilation if no language is specified
}{}


% ******************************************************************************
% 1. Encoding and languages
% ******************************************************************************

% Unicode input and output
\RequirePackage[utf8]{inputenc}
\RequirePackage[T1]{fontenc}

% The desired (main) language is automatically passed from the global options and the second language (english/ngerman) will also be available
\RequirePackage{babel}

% Abbreviations with thin spaces that will never be broken across lines
\newcommand{\ie}{i.\,e.}
\newcommand{\Ie}{I.\,e.}
\newcommand{\eg}{e.\,g.}
\newcommand{\Eg}{E.\,g.}
\newcommand{\zB}{z.\,B.}
\newcommand{\ZB}{Z.\,B.}


% ******************************************************************************
% 2. Title page and metadata
% ******************************************************************************

% Define boolean flags to handle unnumbered single or numbered multiple referee(s)/advisor(s)
\newboolean{numberedreferees}
\setboolean{numberedreferees}{false}
\newboolean{numberedadvisors}
\setboolean{numberedadvisors}{false}

\RequirePackage{uni-titlepage}

% Use file 'title-luh.def' as style definition. Apparently, this file must be placed in the project root.
\TitlePageStyle[extratilepage]{luh}

% Load default metadata (e.g., university, faculty, department, discipline)
\input{internal/metadata}


% ******************************************************************************
% 3. Configuration of classicthesis
% ******************************************************************************
\PassOptionsToPackage{
    % drafting            = true,         % Print version information on the bottom of the pages
    tocaligned          = false,        % Use indentation for the ToC (instead of aligning all items to the left side)
    dottedtoc           = true,         % Page numbers in ToC are flushed right and connected to section headings using dots
    eulerchapternumbers = true,         % Use fancy Euler math font for chapter numbers (otherwise Palatino)
    linedheaders        = false,        % Do not add a horizontal line above the chapter names (only one horizontal line below)
    floatperchapter     = true,         % Numbering of floats per chapter (i.e., Figure 1.1 and Table 4.2)
    eulermath           = false,        % Do not use Euler fonts for mathematical formulae (uses Palatino instead)
    beramono            = true,         % Use Bera Mono as monospaced font
    palatino            = true,         % Use Palatino font as default font
    style               = classicthesis % General style (alternative: arsclassica)
}{classicthesis}  % Note: The package will actually be loaded later


% ******************************************************************************
% 4. Bibliography
% ******************************************************************************
\RequirePackage[
    backend     = biber,        % Default backend with full support for all features
    bibencoding = auto,         % Use same encoding for .bib files as for .tex files (see package inputenc)
    language    = auto,         % Use the 'main' language of babel
    bibstyle    = ieee,         % Use IEEE as bibliography style
    citestyle   = numeric-comp, % Use compact numeric style for citations
    sorting     = none,         % Process all entries in citation order (Alternative: "nyt" = name, year, title)
    natbib      = true,         % Load natbib compatibility module which provides aliases for natbib's citation commands (\citep and \citet)
    maxbibnames = 20,           % Show a maximum of 20 authors for each bibliography entry
    date        = comp,         % Use long date format with compact ranges
    url         = false,        % Hide URLs (except for @online entries)
    doi         = true,         % Show DOIs with the corresponding hyperlink to doi.org
    isbn        = false,        % Hide ISBN/ISSN/ISRN
    eprint      = false         % Hide eprint information
]{biblatex}

% Show "n.d." (no date) or "o.D." (ohne Datum) for bibliography items that do not have a publication date (especially websites)
\renewbibmacro*{date}{%
  \iffieldundef{year}
    {\bibstring{nodate}}
    {\printdate}}

% Add bibliography
\addbibresource{bibliography.bib}


% ******************************************************************************
% 5. Some handy packages
% ******************************************************************************
\RequirePackage[babel,english=american]{csquotes}  % Nice quotes by using \enquote{} (instead of '` and '')

% Math environments
\RequirePackage[sumlimits,intlimits,namelimits]{mathtools}  % Load amsmath with the bugfixes/extensions from mathtools
\RequirePackage{amsthm}
\RequirePackage{amssymb}
\RequirePackage{amstext}
\RequirePackage{bm}
\RequirePackage{textcomp}

% Better localized formatting of dates
\RequirePackage{datetime2}
\ifdefined\uspenglish\DTMlangsetup[en-US]{abbr}\fi
\ifdefined\uspgerman\DTMlangsetup[de-DE]{abbr}\fi

\DTMnewdatestyle{DElong}{
    \renewcommand{\DTMdisplaydate}[4]{
        \DTMtwodigits{##3}.\nobreakspace        % day
        \DTMgermanmonthname{##2}\nobreakspace   % month
        \number##1                              % year
    }
    \renewcommand{\DTMDisplaydate}{\DTMdisplaydate}
}

% More useful packages
\RequirePackage{graphicx}                       % Enhanced support for graphics
\RequirePackage[dvipsnames]{xcolor}             % Definition of custom colors
\definecolor{luh-blue}{cmyk}{1, 0.7, 0, 0}      % #00509B; The following definitions are taken from the corporate design guidelines
\definecolor{luh-green}{cmyk}{0.3, 0, 0.94, 0}  % #C8D317
\definecolor{luh-light-blue}{RGB}{153,185,216}
\definecolor{luh-gray}{RGB}{153,153,153}
\definecolor{luh-light-gray}{RGB}{204,204,204}
\RequirePackage{transparent}                    % Color stack for transparency
\RequirePackage{xspace}                         % Improve automatic spacing after commands by using \xspace
\RequirePackage[printonlyused,smaller]{acronym} % Handling of acronyms
\def\bflabel#1{{\acsfont{#1}\hfill}}            % TODO: Bold acronyms?
\RequirePackage{enumitem}                       % Sub-items in enumerate environment (1.1, 1.2, etc.)
\setlist[enumerate]{label*=\arabic*.}
\RequirePackage{todonotes}                      % Easily visible todo notes
\RequirePackage[newcommands]{ragged2e}          % Definitions of ragged alignment with hyphenation (automatically replace LaTeXs definitions)
\RequirePackage{siunitx}                        % Printing of any physical quantities with correct SI units
\sisetup{mode=match, per-mode=symbol}           % Keep math/text mode as is and use "/" for fractions
\input{misc/siunitx-units}
\RequirePackage{eurosym}                        % Nice looking € symbol
\DeclareUnicodeCharacter{20AC}{\euro}


% ******************************************************************************
% 6. Floats: tables, (sub)figures, and captions
% ******************************************************************************
\RequirePackage{tabularx}                       % Better tables
\setlength{\extrarowheight}{3pt}                % Increase table row height
\newcommand{\tableheadline}[1]{\multicolumn{1}{l}{\spacedlowsmallcaps{#1}}}
\RequirePackage{caption}
\RequirePackage[labelformat=simple]{subcaption} % Allow subfigures and -tables and do not put their number in parentheses
\renewcommand\thesubfigure{(\alph{subfigure})}


% ******************************************************************************
% 7. Code listings
% ******************************************************************************
\RequirePackage{listings}
\lstset{
    captionpos       = b,                           % Show captions below listings
    frame            = single,                      % Show a box around listings
    basicstyle       = \small\ttfamily,             % Basic style that is applied to all code inside a listing
    emphstyle        = \color{luh-blue}\underbar,   % Emphasized word will be blue and underlines
    keywordstyle     = \color{luh-blue},            % Keywords will be blue
    identifierstyle  = \color{black},               % Identifiers will be black
    commentstyle     = \color{luh-gray},            % Comments will be gray
    stringstyle      = \color{luh-green},           % String will be green
    numbers          = left,                        % Display line numbers on the left edge of the listing
    numberstyle      = \scriptsize,                 % Use smaller (but not tiny) font size for line numbers
    stepnumber       = 1,                           % Show line numbers for every line of code
    numbersep        = 8pt,                         % Distance between line numbers and the listing
    showstringspaces = false,                       % Do not show special characters for spaces in strings
    breaklines       = true,                        % Automatically break long lines
    aboveskip        = .75\baselineskip,            % Vertical space above displayed listing
    belowskip        = .50\baselineskip,            % Vertical space below displayed listing
    belowcaptionskip = .25\baselineskip             % Vertical space below caption
}


% ******************************************************************************
% 8. Actually load classicthesis
% ******************************************************************************
\RequirePackage{classicthesis}

% Replace color definitions of classicthesis with LUH colors
\colorlet{CTurl}{luh-blue}
\colorlet{CTlink}{luh-blue}
\colorlet{CTtitle}{luh-blue}
\colorlet{CTsemi}{luh-light-blue}
\colorlet{CTcitation}{luh-blue}


% ******************************************************************************
% 9. Fine-tune hyperreferences (hyperref should be called last)
% ******************************************************************************

\hypersetup{
    final,                              % Enable all options (opposite of 'draft')
    colorlinks          = false,        % Do not use colored links (they would also be colored when printing)
    urlcolor            = CTurl,        % Color for linked URLs
    linkcolor           = CTlink,       % Color for normal internal links
    citecolor           = CTcitation,   % Color for bibliographical citations in text
    pdfborder           = {0 0 0},      % No colored borders around hyperlinks
    breaklinks          = true,         % Allow links to be broken
    pageanchor          = true,         % Create link targets for each page (necessary for some other options)
    plainpages          = false,        % Do not use arabic forms for all page anchors (this would cause duplicate anchors)
    linktoc             = all,          % Both section names and page numbers of the ToC are hyperlinks
    hyperfootnotes      = true,         % Footnote marks link to the footnote text (does not work for some reason)
    bookmarks           = true,         % Produce metadata bookmarks for PDF viewers
    bookmarksnumbered   = true,         % Include Section numbers in the bookmarks
    bookmarksopen       = true,         % Open/show the bookmarks by default when the PDF is opened in a supporting application
    bookmarksopenlevel  = 1,            % Expand only the first level of bookmarks by default (other levels are collapsed)
    hypertexnames       = true,         % Use guessable names for links (e.g., 'page.iv', 'page.27', etc.)
    pdfhighlight        = /I,           % Inverse highlight when clicking on links
    pdfpagemode         = UseOutlines,  % Open bookmarks by default
    pdfstartpage        = 1,            % Which page to display first when opening the PDF file
    pdfstartview        = Fit,          % Fit the whole page to the window
    pdfpagelayout       = TwoPageRight  % Display two pages at once with odd pages on the right
}

% Metadata like author and title is only finally set in the main file before \begin{document}
\AtEndPreamble{
    \hypersetup{
        pdftitle            = {\usetitleelement{title}},
        pdfauthor           = {\usetitleelement{author}},
        pdfsubject          = {\usetitleelement{subject}},
        pdfkeywords         = {},
        pdfcreator          = {LaTeX with hyperref and classicthesis}
    }
    
    \issuedate{\DTMusedate{issuedate}}
    \submissiondate{\DTMusedate{submissiondate}}
}

% Use cleveref package for improved references in text using either \cref{} or \Cref{}
\RequirePackage[nameinlink,noabbrev]{cleveref}
\creflabelformat{equation}{#2#1#3}      % No parentheses aroud equation numbers

% Fix some translations manually
\ifdefined\uspenglish
    \newcommand*{\declarationname}{Declaration}
    \renewcommand*{\lstlistlistingname}{List of Listings}
    \newcommand*{\listacronymname}{Acronyms}
\fi
\ifdefined\uspgerman
    \newcommand*{\declarationname}{Eidesstattliche Erklärung}
    \renewcommand*{\lstlistlistingname}{Codeverzeichnis}
    \renewcommand*{\lstlistingname}{Codebeispiel}
    \crefname{listing}{Codebeispiel}{Codebeispiele}
    \newcommand*{\listacronymname}{Akronyme}
\fi

% Hide labels of floats in their respective lists (i.e., skip "Figure", "Table" and "Listing")
% Redefinitions are done using package tocloft. For more information see:
% https://tex.stackexchange.com/questions/689564/in-classicthesis-is-it-possible-to-have-lists-of-figures-listings-etc-without
% https://tex.stackexchange.com/questions/235191/increase-space-between-numbers-and-text-in-list-of-figures
\renewcommand{\cftfigpresnum}{}
\settowidth{\figurelabelwidth}{999}
\cftsetindents{figure}{0em}{\figurelabelwidth}
\setlength{\cftfignumwidth}{3em}
\renewcommand{\cfttabpresnum}{}
\settowidth{\tablelabelwidth}{999}
\cftsetindents{table}{0em}{\tablelabelwidth}
\setlength{\cfttabnumwidth}{3em}
\renewcommand{\cftlistingspresnum}{}
\settowidth{\listingslabelwidth}{999}
\cftsetindents{listings}{0em}{\listingslabelwidth}
\setlength{\cftlistingsnumwidth}{3em}


\RequirePackage{scrhack}  % Fix behavior of some packages in combination with KOMA-Script classes